from typing import Literal, Self

import lox_space as lox
from pydantic import Field, PrivateAttr

from ephemerista import BaseModel

type Frame = Literal[
    "ICRF",
    "IAU_SUN",
    "IAU_MERCURY",
    "IAU_VENUS",
    "IAU_EARTH",
    "IAU_MARS",
    "IAU_JUPITER",
    "IAU_SATURN",
    "IAU_URANUS",
    "IAU_NEPTUNE",
    "IAU_PLUTO",
    "IAU_MOON",
    "IAU_PHOBOS",
    "IAU_DEIMOS",
    "IAU_IO",
    "IAU_EUROPA",
    "IAU_GANYMEDE",
    "IAU_CALLISTO",
    "IAU_AMALTHEA",
    "IAU_HIMALIA",
    "IAU_ELARA",
    "IAU_PASIPHAE",
    "IAU_SINOPE",
    "IAU_LYSITHEA",
    "IAU_CARME",
    "IAU_ANANKE",
    "IAU_LEDA",
    "IAU_THEBE",
    "IAU_ADRASTEA",
    "IAU_METIS",
    "IAU_CALLIRRHOE",
    "IAU_THEMISTO",
    "IAU_MAGACLITE",
    "IAU_TAYGETE",
    "IAU_CHALDENE",
    "IAU_HARPALYKE",
    "IAU_KALYKE",
    "IAU_IOCASTE",
    "IAU_ERINOME",
    "IAU_ISONOE",
    "IAU_PRAXIDIKE",
    "IAU_AUTONOE",
    "IAU_THYONE",
    "IAU_HERMIPPE",
    "IAU_AITNE",
    "IAU_EURYDOME",
    "IAU_EUANTHE",
    "IAU_EUPORIE",
    "IAU_ORTHOSIE",
    "IAU_SPONDE",
    "IAU_KALE",
    "IAU_PASITHEE",
    "IAU_HEGEMONE",
    "IAU_MNEME",
    "IAU_AOEDE",
    "IAU_THELXINOE",
    "IAU_ARCHE",
    "IAU_KALLICHORE",
    "IAU_HELIKE",
    "IAU_CARPO",
    "IAU_EUKELADE",
    "IAU_CYLLENE",
    "IAU_KORE",
    "IAU_HERSE",
    "IAU_DIA",
    "IAU_MIMAS",
    "IAU_ENCELADUS",
    "IAU_TETHYS",
    "IAU_DIONE",
    "IAU_RHEA",
    "IAU_TITAN",
    "IAU_HYPERION",
    "IAU_IAPETUS",
    "IAU_PHOEBE",
    "IAU_JANUS",
    "IAU_EPIMETHEUS",
    "IAU_HELENE",
    "IAU_TELESTO",
    "IAU_CALYPSO",
    "IAU_ATLAS",
    "IAU_PROMETHEUS",
    "IAU_PANDORA",
    "IAU_PAN",
    "IAU_YMIR",
    "IAU_PAALIAQ",
    "IAU_TARVOS",
    "IAU_IJIRAQ",
    "IAU_SUTTUNGR",
    "IAU_KIVIUQ",
    "IAU_MUNDILFARI",
    "IAU_ALBIORIX",
    "IAU_SKATHI",
    "IAU_ERRIAPUS",
    "IAU_SIARNAQ",
    "IAU_THRYMR",
    "IAU_NARVI",
    "IAU_METHONE",
    "IAU_PALLENE",
    "IAU_POLYDEUCES",
    "IAU_DAPHNIS",
    "IAU_AEGIR",
    "IAU_BEBHIONN",
    "IAU_BERGELMIR",
    "IAU_BESTLA",
    "IAU_FARBAUTI",
    "IAU_FENRIR",
    "IAU_FORNJOT",
    "IAU_HATI",
    "IAU_HYRROKKIN",
    "IAU_KARI",
    "IAU_LOGE",
    "IAU_SKOLL",
    "IAU_SURTUR",
    "IAU_ANTHE",
    "IAU_JARNSAXA",
    "IAU_GREIP",
    "IAU_TARQEQ",
    "IAU_AEGAEON",
    "IAU_ARIEL",
    "IAU_UMBRIEL",
    "IAU_TITANIA",
    "IAU_OBERON",
    "IAU_MIRANDA",
    "IAU_CORDELIA",
    "IAU_OPHELIA",
    "IAU_BIANCA",
    "IAU_CRESSIDA",
    "IAU_DESDEMONA",
    "IAU_JULIET",
    "IAU_PORTIA",
    "IAU_ROSALIND",
    "IAU_BELINDA",
    "IAU_PUCK",
    "IAU_CALIBAN",
    "IAU_SYCORAX",
    "IAU_PROSPERO",
    "IAU_SETEBOS",
    "IAU_STEPHANO",
    "IAU_TRINCULO",
    "IAU_FRANCISCO",
    "IAU_MARGARET",
    "IAU_FERDINAND",
    "IAU_PERDITA",
    "IAU_MAB",
    "IAU_CUPID",
    "IAU_TRITON",
    "IAU_NEREID",
    "IAU_NAIAD",
    "IAU_THALASSA",
    "IAU_DESPINA",
    "IAU_GALATEA",
    "IAU_LARISSA",
    "IAU_PROTEUS",
    "IAU_HALIMEDE",
    "IAU_PSAMATHE",
    "IAU_SAO",
    "IAU_LAOMEDEIA",
    "IAU_NESO",
    "IAU_CHARON",
    "IAU_NIX",
    "IAU_HYDRA",
    "IAU_KERBEROS",
    "IAU_STYX",
    "IAU_GASPRA",
    "IAU_IDA",
    "IAU_DACTYL",
    "IAU_CERES",
    "IAU_PALLAS",
    "IAU_VESTA",
    "IAU_PSYCHE",
    "IAU_LUTETIA",
    "IAU_KLEOPATRA",
    "IAU_EROS",
    "IAU_DAVIDA",
    "IAU_MATHILDE",
    "IAU_STEINS",
    "IAU_BRAILLE",
    "IAU_WILSON_HARRINGTON",
    "IAU_TOUTATIS",
    "IAU_ITOKAWA",
    "IAU_BENNU",
]


class ReferenceFrame(BaseModel):
    abbreviation: Frame = Field(default="ICRF")
    _frame: lox.Frame = PrivateAttr()

    def __init__(self, frame: lox.Frame | None = None, **data):
        super().__init__(**data)

        if not frame:
            self._frame = lox.Frame(self.abbreviation)

    @classmethod
    def _from_lox(cls, frame: lox.Frame) -> Self:
        return cls(abbreviation=frame.abbreviation(), _frame=frame)
